@@ -7,7 +7,7 @@ type PagerModeNotFound struct {
 }
 
 func (m PagerModeNotFound) drawFooter(_ string, _ string) {
-	m.pager.setFooter("Not found: " + m.pager.searchString)
+	m.pager.setFooter("Not found: "+m.pager.searchString, "")
 }
 
 func (m PagerModeNotFound) onKey(key twin.KeyCode) {
